use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007000
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
AIX
BENCHMARKING
CRLF
SHA
NFS
canonpath
codepoints
cwd
dirname
fatalize
lstat
mkpath
opena
openr
openrw
openw
realpath
stringifying
subclasses
touchpath
UNC
unlinked
utf
David
Golden
dagolden
Alex
Efros
powerman
Chris
Williams
bingos
xdg
Steinbrunner
dsteinbrunner
Doug
Bell
madcityzen
Gabor
Szabo
szabgab
Gabriel
Andrade
gabiruh
George
Hartzell
hartzell
Geraud
Continsouzas
geraud
Goro
Fuji
gfuji
Graham
Knop
haarg
James
Hunt
james
Karen
Etheridge
ether
Mark
Ellis
mark
Martin
Kjeldsen
mk
Michael
Schwern
mschwern
Philippe
Bruhat
BooK
book
Regina
Verbae
regina
Roy
Ivy
III
RIVY
Shlomi
Fish
shlomif
Smylers
Tatsuhiko
Miyagawa
miyagawa
Toby
Inkster
tobyink
Yanick
Champoux
yanick
김도형
Keedi
Kim
keedi
lib
Path
Tiny
